"use strict";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

Object.defineProperty(exports, "__esModule", { value: true });
var event_1 = require("./event");
var tag = "ascoders-dob";
var globalOrWindow = (typeof self === "undefined" ? "undefined" : _typeof(self)) === "object" && self.self === self && self || (typeof global === "undefined" ? "undefined" : _typeof(global)) === "object" && global.global === global && global || undefined;

var GlobalState = function GlobalState() {
  _classCallCheck(this, GlobalState);

  /**
   * 存储所有代理
   * key：代理 + 原始对象
   */
  this.proxies = new WeakMap();
  /**
   * 所有代理 -> 原始对象的映射
   */
  this.originObjects = new WeakMap();
  /**
   * 存储所有要代理原始的对象
   * 以对象每个 key 存储监听事件
   */
  this.objectReactionBindings = new WeakMap();
  /**
   * 当前 reaction
   */
  this.currentReaction = null;
  /**
   * 批量执行深入，比如每次调用 runInAction 深入 +1，调用完 -1，深入为 0 时表示执行完了
   * 当 batchDeep === 0 时，表示操作队列完毕
   */
  this.batchDeep = 0;
  /**
   * 所有 action 中等执行队列的集合
   */
  this.pendingReactions = new Set();
  /**
   * 忽略动态对象的 symbol
   */
  this.ignoreDynamicSymbol = Symbol();
  /**
   * track 嵌套 track 时，临时缓存下来的 track 队列，等待上一层 track 执行完后再执行
   */
  this.pendingTacks = new Set();
  /**
   * 是否开启 debug
   */
  this.useDebug = false;
  /**
   * 当前正在操作的 debugOutputAction 对象，这样在修改值的时候，直接操作其 changeList 队列即可
   */
  this.currentDebugOutputAction = null;
  /**
   * 各 batchDeep 的根 debugOutputBundleAction
   */
  this.debugOutputActionMapBatchDeep = new Map();
  /**
   * object 的父级信息
   */
  this.parentInfo = new WeakMap();
  /**
   * 当前所在 debugName（由 decorator Action 触发）
   */
  this.currentDebugName = null;
  /**
   * 当前所在 debugId
   */
  this.currentDebugId = null;
  /**
   * 当前是否处于严格模式
   */
  this.strictMode = false;
  /**
   * 事件
   */
  this.event = new event_1.Event();
  /**
   * 唯一 id 计数器
   */
  this.uniqueIdCounter = 0;
};

var globalState = new GlobalState();
exports.globalState = globalState;
if (globalOrWindow[tag]) {
  exports.globalState = globalState = globalOrWindow[tag];
} else {
  globalOrWindow[tag] = globalState;
}
//# sourceMappingURL=global-state.js.map