"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

Object.defineProperty(exports, "__esModule", { value: true });
var global_state_1 = require("./global-state");
var utils_1 = require("./utils");

var Reaction = function () {
    function Reaction(name, callback, delay) {
        _classCallCheck(this, Reaction);

        // 执行延时
        this.delay = null;
        // 与自身绑定的对象列表
        this.keyBinders = new Set();
        this.name = name;
        this.callback = callback;
        this.delay = delay;
    }
    /**
     * 在 track 中访问的变量，都会与当前 reaction 绑定，当这些变量产生修改时，会触发
     * @TODO: 目前仅支持同步，还未找到支持异步的方案！
     */


    _createClass(Reaction, [{
        key: "track",
        value: function track(callback) {
            // 如果已经在 track 中，那就在待执行队列直接新增一项，返回
            // 上一层 runObserver 执行完后会执行待执行队列
            if (utils_1.inTrack()) {
                global_state_1.globalState.pendingTacks.add(this.track.bind(this, callback));
                return;
            }
            /**
             * 执行 callback
             */
            global_state_1.globalState.currentReaction = this;
            // 先清空所有绑定
            this.clearBinding();
            try {
                callback({
                    debugId: global_state_1.globalState.currentDebugId
                });
            } finally {
                global_state_1.globalState.currentReaction = null;
            }
            /**
             * 执行 track 剩余队列
             */
            // 队列执行次数
            var currentRunCount = 0;
            var MAX_RUN_COUNT = 1000;
            // 复制一份队列，防止嵌套 observe 时，不断在 nestedObserverQueue 添加 observer 导致死循环
            // 复制完后，原队列清空，让执行时加在干净的新队列
            var pedingTacksCopy = Array.from(global_state_1.globalState.pendingTacks);
            global_state_1.globalState.pendingTacks.clear();
            pedingTacksCopy.forEach(function (eachTack) {
                currentRunCount++;
                if (currentRunCount >= MAX_RUN_COUNT) {
                    // tslint:disable-next-line:no-console
                    console.warn("执行次数达到上限，可能存在死循环");
                    global_state_1.globalState.pendingTacks.clear();
                    return;
                }
                eachTack();
            });
        }
        /**
         * 销毁
         */

    }, {
        key: "dispose",
        value: function dispose() {
            this.clearBinding();
            this.callback = null;
        }
        /**
         * 触发 reaction 的 observe
         */

    }, {
        key: "run",
        value: function run() {
            // 简单的执行回调函数即可
            if (this.callback) {
                this.callback();
            }
        }
        /**
         * 添加绑定者（某 object 的 reactionsForKey）
         */

    }, {
        key: "addBinder",
        value: function addBinder(keyBinder) {
            this.keyBinders.add(keyBinder);
        }
        /**
         * 清空绑定
         */

    }, {
        key: "clearBinding",
        value: function clearBinding() {
            var _this = this;

            // 清空所有 binder 对自己引用
            // 再清空存储的 binders
            this.keyBinders.forEach(function (keyBinder) {
                keyBinder.delete(_this);
            });
            this.keyBinders.clear();
        }
    }]);

    return Reaction;
}();

exports.Reaction = Reaction;
//# sourceMappingURL=reaction.js.map