"use strict";

Object.defineProperty(exports, "__esModule", { value: true });
var global_state_1 = require("./global-state");
/**
 * 是否是基本类型
 */
function isPrimitive(value) {
    if (value === null || value === undefined) {
        return true;
    }
    if (typeof value === "string" || typeof value === "number" || typeof value === "boolean" || value instanceof Date) {
        return true;
    }
    return false;
}
exports.isPrimitive = isPrimitive;
/**
 * 空函数
 */
// tslint:disable-next-line:no-empty
exports.noop = function () {};
/**
 * 是否在 action 中
 */
function inAction() {
    return global_state_1.globalState.batchDeep !== 0;
}
exports.inAction = inAction;
/**
 * 是否在 track 中
 */
function inTrack() {
    return !!global_state_1.globalState.currentReaction;
}
exports.inTrack = inTrack;
/**
 * 找到某个对象、键值的 binder
 */
function getBinder(object, key) {
    var keysForObject = global_state_1.globalState.objectReactionBindings.get(object);
    if (!keysForObject) {
        keysForObject = new Map();
        global_state_1.globalState.objectReactionBindings.set(object, keysForObject);
    }
    var reactionsForKey = keysForObject.get(key);
    if (!reactionsForKey) {
        reactionsForKey = new Set();
        keysForObject.set(key, reactionsForKey);
    }
    return {
        binder: keysForObject,
        keyBinder: reactionsForKey
    };
}
exports.getBinder = getBinder;
/**
 * 开启调试模式
 */
function startDebug() {
    global_state_1.globalState.useDebug = true;
}
exports.startDebug = startDebug;
/**
 * 关闭调试模式
 */
function stopDebug() {
    global_state_1.globalState.useDebug = false;
}
exports.stopDebug = stopDebug;
/**
 * 设置处于严格模式
 */
function useStrict() {
    global_state_1.globalState.strictMode = true;
}
exports.useStrict = useStrict;
/**
 * 取消严格模式
 */
function cancelStrict() {
    global_state_1.globalState.strictMode = false;
}
exports.cancelStrict = cancelStrict;
/**
 * 生成随机唯一 id
 */
function getUniqueId() {
    return global_state_1.globalState.uniqueIdCounter++;
}
exports.getUniqueId = getUniqueId;
//# sourceMappingURL=utils.js.map